package com.om.mxs.client.japi.samples;

/*
 * ContentSearch.java
 *
 * Version 4.2.0.17
 *
 * Copyright (c) 2003-2010 Object Matrix, Ltd. All rights reserved.
 * 
 * Redistribution and use of MatrixStore client API software in source and
 * binary forms ("Software"), with or without modification,
 * is permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright statements
 * and notices, this list of conditions and the following disclaimer. 
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * 3. The name "MatrixStore" and "Object Matrix Limited" must not be used to endorse
 * or promote products derived from this Software without prior written permission of
 * Object Matrix Ltd. For written permission, please contact info@object-matrix.com
 * 
 * 4. Products derived from this Software may only utilise this Software to connect to and to 
 * communicate with servers running MatrixStore server software from Object Matrix Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED BY OBJECT MATRIX AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OBJECT MATRIX OR ITS
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 */


import java.io.IOException;
import java.io.OutputStream;
import java.nio.charset.Charset;
import java.util.Set;

import com.om.mxs.client.japi.Attribute;
import com.om.mxs.client.japi.Constants;
import com.om.mxs.client.japi.MatrixStore;
import com.om.mxs.client.japi.MatrixStoreConnection;
import com.om.mxs.client.japi.MxsObject;
import com.om.mxs.client.japi.ObjectTypedAttributeView;
import com.om.mxs.client.japi.Vault;
import com.om.mxs.client.japi.cred.Credentials;

/**
 * This example shows how to search for objects stored in a MatrixStore server with a
 * more complex search term.
 *
 * This example assumes string key and value although the system allows utf keys and
 * any value.
 *
 * The resultant list of object identifiers (OIDs) returned by MatrixStore can be used
 * to access the objects if desired (see RetrieveObject example).
 *
 * For all MatrixStore concepts not explained here please reference
 * the MatrixStore Java API User Guide and the Java API documentation.
 *
 */
public class ContentSearch {

    protected static final String GS = "\u241D"; // Group Separator

    public static void main(String[] args){

        if (args.length != 2){

            usage(null);
            System.exit(-1);
        }

        // Initialise the variables required to run this example
        // taking the values from the argument line
        
        // Cluster identifier
        String clusterId= args[0];
        // Comma separated list of IP addresses
        String address = args[1];
        
        // Reference to vault connection
        MatrixStoreConnection conn = null;
    	MatrixStore mxs = null;
        Vault vault = null;

        try{

            // Create a connection to a MatrixStore vault. 
            // IMPORTANT: 
            // The vault you are connecting should have Content Search enabled otherwise nothing will work.
        	// Content Search can be enabled in Web Admin.
        	String[] addresses = address.split(",");
        	conn = MatrixStoreConnection.builder().withHosts(addresses).withClusterId(clusterId).build();
        	Credentials creds = Credentials.newUsernamePasswordCredentials("testuser", "password");
        	mxs = MatrixStore.builder().withConnection(conn).withCredentials(creds).build();
            vault = mxs.openVault("7c0e47e4-bde2-11e9-a975-cd0d9f8facdc");           

            MxsObject object1 = vault.getObject(null);
            // create a view to the metadata
            ObjectTypedAttributeView metadata = object1.getAttributeView();
            // add metadata after creation
            
            // IMPORTANT:
            // this attribute triggers content indexing for this object.
            // It has to be added at the creation of the object. It cannot be added later 
            // Content search can always be performed on metadata attributes
            metadata.writeString(Constants.CONTENT_TYPE, Constants.TEXT);
            metadata.writeInt("incrementingInt", (int)10000, true); 
            metadata.writeString("titleString", "A Clash of Kings");
            metadata.writeText("titleText", "A Clash of Kings");
            metadata.writeBoolean("isTrue", true);
            metadata.writeBoolean("isFalse", false);
            OutputStream outStream = object1.newOutputStream();
            outStream.write("five four three two one ".getBytes(Charset.forName("UTF8")));
            outStream.close();
          
            // If you are going to search on content, you need to wait until
            // the indexer has gone through the content
            System.out.println("Wait till the object is indexed");
            while (!object1.isIndexed()){
                try{
                    Thread.sleep(5000);
                }catch (InterruptedException e){}
            }
            
            // Search for the first object
            Set<String> ids = null;

            /*
             * Exact Matching 
             */
            // find all objects with “three” in the content 
            ids = contentSearch(vault, "three");
            System.out.println("Exact Matching found " + ids.size() + " objects");

            // search all objects with exact matching of attribute incrementingInt = 10005
            ids = contentSearch(vault, "*\nrange:incrementingInt"+ GS +"int"+ GS +">="+ GS +"10000"+ GS +"<="+ GS +"10000");
            System.out.println("Exact Matching found " + ids.size() + " objects");
  
            /*
             * Range Matching 
             */
            // search all objects with attribute incrementingInt between 100 and 20000
            ids = contentSearch(vault, "*\nrange:incrementingInt"+ GS +"int"+ GS +">="+ GS +"100"+ GS +"<="+ GS +"20000");
            System.out.println("Range Matching found " + ids.size() + " objects");

            // V3.2 syntax
            ids = contentSearch(vault, "_mxs_int_incrementingInt: [100 TO 20000]");
            System.out.println("Range Matching with prefix found " + ids.size() + " objects");

            /*
             * Field Sorting
             */
            // find all objects with “two” in the content, sort results by __mxs__id 
            ids = contentSearch(vault, "two\nsort:>"+ GS +"__mxs__id"+ GS +"string");
            System.out.println("Field Sorting found " + ids.size() + " objects");

            // find all objects with “one” in the content, sort the results on an integer field called “incrementingInt”
            ids = contentSearch(vault, "one\nsort:>"+ GS +"incrementingInt"+ GS +"int");
            
            /*
             * Field Search (Text attribute)
             */
            ids = contentSearch(vault, "titleText:\"A Clash of Kings\"");
            System.out.println("Field Search (Text) found " + ids.size() + " objects");
            
            ids = contentSearch(vault, "titleText:\"kings\"");
            System.out.println("Field Search (Text) found " + ids.size() + " objects");
            
            /*
             * Field Search (String attribute), V3.2
             */
            ids = contentSearch(vault, "_mxs_string_titleString: \"A Clash of Kings\"");
            System.out.println("Field Search (String) with exact match and case-sensitive found " + ids.size() + " objects");
            
            ids = contentSearch(vault, "_mxs_string_titleString: A*");
            System.out.println("Field Search (String)  with wildcard found " + ids.size() + " objects");
            
            ids = contentSearch(vault, "titleString: \"a clash of kings\"");
            System.out.println("Field Search (String) with case-insensitive found " + ids.size() + " objects");
            
            ids = contentSearch(vault, "titleString: \"kings\"");
            System.out.println("Field Search (String) with no exact match found " + ids.size() + " objects");

            /*
             * Field Search (Int attribute), V3.2
             */
            ids = contentSearch(vault, "incrementingInt:10000");
            System.out.println("Field Search (int) found " + ids.size() + " objects");

            ids = contentSearch(vault, "_mxs_int_incrementingInt:10000");
            System.out.println("Field Search with prefix (int) found " + ids.size() + " objects");

            /*
             * Field Search (Boolean attribute), V3.2
             */
            ids = contentSearch(vault, "isTrue:1");
            System.out.println("Field Search (boolean) found " + ids.size() + " objects");

            ids = contentSearch(vault, "isTrue:0");
            System.out.println("Field Search (boolean) found " + ids.size() + " objects");

            ids = contentSearch(vault, "_mxs_boolean_isTrue:true");
            System.out.println("Field Search with prefix (boolean) found " + ids.size() + " objects");

            ids = contentSearch(vault, "_mxs_boolean_isTrue:false");
            System.out.println("Field Search with prefix (boolean) found " + ids.size() + " objects");

            ids = contentSearch(vault, "_mxs_boolean_isFalse:false");
            System.out.println("Field Search with prefix (boolean) found " + ids.size() + " objects");
               
            /*
             * Keywords
             */
            ids = contentSearch(vault, "five\nkeywords:__mxs__id,__mxs__location");
            for (String id: ids) {
                System.out.println("Result with keywords embedded: " + id);
            }

            /*
             * Wildcard Search
             */
            ids = contentSearch(vault, "five*");
            System.out.println("Wildcard Search found " + ids.size() + " objects");

            /*
             * Fuzzy Search
             */
            ids = contentSearch(vault, "fiva~");
            System.out.println("Fuzzy Search found " + ids.size() + " objects");

            /*
             * AND Search
             */
            createSimpleObject(vault, "ANDSearches0 ANDSearches1");
            while (!object1.isIndexed()){
                try{
                    Thread.sleep(5000);
                }catch (InterruptedException e){}
            }

            ids = contentSearch(vault, "*\nANDSearches0 AND ANDSearches1");
            System.out.println("AND Search found " + ids.size() + " objects");
            
            /*
             * OR Search
             */
            createSimpleObject(vault, "ORSearches0");
            createSimpleObject(vault, "ORSearches1");
            ids = contentSearch(vault, "*\nORSearches0 OR ORSearches1");
            System.out.println("OR Search found " + ids.size() + " objects");

            // OR with range, V3.2
            ids = contentSearch(vault, "_mxs_int_incrementingInt: [100 TO 20000] OR titleString: kings");
            System.out.println("OR with range found " + ids.size() + " objects");

            /*
             * NOT Search
             */
            createSimpleObject(vault, "noSearches");
            createSimpleObject(vault, "noSearches MatrixStore");
            ids = contentSearch(vault, "noSearches NOT MatrixStore");
            System.out.println("NOT Search found " + ids.size() + " objects");

            /*
             * Grouping
             */
            createSimpleObject(vault, "Grouping0");
            createSimpleObject(vault, "Grouping1 MatrixStore");
            ids = contentSearch(vault, "((Grouping0 OR Grouping1) AND MatrixStore)");
            System.out.println("Grouping found " + ids.size() + " objects");
            
        }catch (IOException e){
 
            // A MatrixStore error code will be returned along with a message
            // explaining the meaning of the error.
            usage("MatrixStore Error: code=" + Vault.getErrorCode(e) +
                  " message=" + e.getMessage());

        }catch (Exception e){

            usage("Error: " + e.getMessage());
            
        }finally{
            
            // Close all the remaining objects
        	if (vault != null)
                vault.dispose();
            if (mxs != null)
            	mxs.dispose();
            if (conn != null)
            	conn.close();
        }
    }

    private static Set<String> contentSearch(Vault vault, String query) throws IOException {
        Attribute contentQuery = new Attribute(Constants.CONTENT, query);
        return vault.searchObjects(contentQuery, 10);
    }

    private static String createSimpleObject(Vault vault, String content) throws IOException {
        MxsObject object = null;
        object = vault.getObject(null);
        // create a view to the metadata
        ObjectTypedAttributeView metadata = object.getAttributeView();
        metadata.writeString(Constants.CONTENT_TYPE, Constants.TEXT);
        OutputStream outStream = object.newOutputStream();
        outStream.write(content.getBytes(Charset.forName("UTF8")));
        outStream.close();
        String id = object.getId();
        // wait till it is indexed
        while (!object.isIndexed()){
            try{
                Thread.sleep(3000);
            }catch (InterruptedException e){}
        }
        return id;
    }

    private static void usage (String msg){

        if (msg != null){
            System.err.println (msg);
        }
        System.err.println ("Usage: java ContentSearch clusterId address");
    }    

}
