package com.om.mxs.client.japi.samples;

/**
 * StoreObjectWithChannel.java
 *
 * Version 4.2.0.17
 *
 * Copyright (c) 2003-2014 Object Matrix, Ltd. All rights reserved.
 * 
 * Redistribution and use of MatrixStore client API software in source and
 * binary forms ("Software"), with or without modification,
 * is permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright statements
 * and notices, this list of conditions and the following disclaimer. 
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * 3. The name "MatrixStore" and "Object Matrix Limited" must not be used to endorse
 * or promote products derived from this Software without prior written permission of
 * Object Matrix Ltd. For written permission, please contact info@object-matrix.com
 * 
 * 4. Products derived from this Software may only utilise this Software to connect to and to 
 * communicate with servers running MatrixStore server software from Object Matrix Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED BY OBJECT MATRIX AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OBJECT MATRIX OR ITS
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 */

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.ByteChannel;
import java.nio.channels.FileChannel;

import com.om.mxs.client.japi.AccessOption;
import com.om.mxs.client.japi.MatrixStore;
import com.om.mxs.client.japi.MatrixStoreConnection;
import com.om.mxs.client.japi.MxsObject;
import com.om.mxs.client.japi.ObjectTypedAttributeView;
import com.om.mxs.client.japi.Vault;
import com.om.mxs.client.japi.cred.Credentials;

/**
 * This example shows how to store data and metadata in MatrixStore.
 *
 * The object Identifier (oid) returned by MatrixStore must be used to access
 * the object afterwards (shown in other examples).
 *
 * For all MatrixStore concepts not explained here please reference
 * the MatrixStore Java API User Guide and the Java API documentation.
 *
 */
public class StoreObjectWithChannel {

    public static void main(String[] args) throws IOException {

        if (args.length != 3){
            usage(null);
            System.exit(-1);
        }

        // Initialise the variables required to run this example
        // taking the values from the argument line

        // Cluster identifier
        String clusterId= args[0];
        // Comma separated list of IP addresses 
        String address= args[1];
        // Full path to location of file to write
        String fileName= args[2];
        
        // Reference to vault connection
    	MatrixStoreConnection conn = null;
    	MatrixStore mxs = null;
        Vault vault = null; 
        
        FileInputStream in = null;
        try{
            // Open the file input stream on the file passed in as an argument
            in = new FileInputStream(fileName);
            FileChannel inChannel = in.getChannel();

            // Create a connection to a MatrixStore vault. 
        	String[] addresses = address.split(",");
        	conn = MatrixStoreConnection.builder().withHosts(addresses).withClusterId(clusterId).build();
        	Credentials creds = Credentials.newUsernamePasswordCredentials("testuser", "password");
        	mxs = MatrixStore.builder().withConnection(conn).withCredentials(creds).build();
            vault = mxs.openVault("7c0e47e4-bde2-11e9-a975-cd0d9f8facdc");   
            
            // Create an object instance. 
            // null object id means that the object is in memory and not in the server yet
            MxsObject object = vault.getObject(null);
            // create a view to the metadata
            ObjectTypedAttributeView metadata = object.getAttributeView();
            // add metadata
            String value0= "Bristol";
            metadata.writeString("Area", value0);
            String value1= "Post-Processed";
            metadata.writeString("ImageType", value1);            
            int value2 = 10;
            metadata.writeInt("Code", value2);       
            // add basic attributes for DropSpot and MXFS compatibility
            metadata.writeString("MXFS_PARENTOID", "");            
            metadata.writeBoolean("MXFS_INTRASH", false);            
            metadata.writeString("MXFS_FILENAME", new File(fileName).getName());            
            metadata.writeInt("MXFS_CATEGORY", 0);            

            // Open a byte channel, i.e. we are creating
            // a new MatrixStore object.
            ByteChannel outChannel = object.newObjectChannel(AccessOption.CREATE);

            // Create a byte buffer to transfer the data. 
            // A good buffer size for transmissions is 64KB segments.
            ByteBuffer data = ByteBuffer.allocate(64 * 1024);

            // Read the data from the file and send it to the server
            while ((inChannel.read(data)) != -1){
                data.flip();
                outChannel.write(data);
                data.clear();
            }

            // Close the channels
            inChannel.close();
            outChannel.close();

            // Get the object Identifier (OID) from the newly create object. It is
            // important to keep the OID in a safe place as it is the reference to
            // your data.
            String objectId = object.getId();

            System.out.println ("The identifier of the resulting MatrixStore object is "
                                + objectId);
            
        }catch (IOException e){

            // A MatrixStore error code will be returned along with a message 
            // explaining the meaning of the error.
            usage("MatrixStore Error: code=" + Vault.getErrorCode(e) +
                  " message=" + e.getMessage());

        }catch (Exception e){

            usage("Error: " + e.getMessage());
            
        }finally{
            if (in != null)
            	in.close();
        	
            // Close all the remaining instances
        	if (vault != null)
                vault.dispose();
            if (mxs != null)
            	mxs.dispose();
            if (conn != null)
            	conn.close();
        }
    }

    private static void usage (String msg){

        if (msg != null){
            System.err.println (msg);
        }
        System.err.println ("Usage: java StoreObjectWithChannel clusterId address fileName");
    }

}
